from typing import Final

URL_OADS: Final[str] = "https://eocat.esa.int/eo-catalogue"
URL_MAAP: Final[str] = "https://catalog.maap.eo.esa.int/catalogue"
URL_ENTRYPOINT: Final[str] = URL_MAAP
URL_AUTHENTIFICATION: Final[str] = "https://eoiam-idp.eo.esa.int/samlsso"

PROGRAM_NAME: Final[str] = "ecdownload"
PROGRAM_DESCRIPTION: Final[str] = """description is missing"""
PROGRAM_SETUP_INSTRUCTIONS: Final[str] = """setup instructions are missing"""
