from typing import Final

# WGS 84
SEMI_MAJOR_AXIS_METERS: Final[float] = 6378137.0
SEMI_MINOR_AXIS_METERS: Final[float] = 6356752.314245

MEAN_EARTH_RADIUS_METERS = (SEMI_MAJOR_AXIS_METERS * 2 + SEMI_MINOR_AXIS_METERS) / 3
